/****************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/

package org.apache.james.vault.metadata;

import static org.apache.james.vault.metadata.DeletedMessageVaultMetadataFixture.STORAGE_INFORMATION;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import org.apache.james.vault.DeletedMessageFixture;
import org.junit.jupiter.api.Test;

import nl.jqno.equalsverifier.EqualsVerifier;

class DeletedMessageWithStorageInformationTest {

    @Test
    void shouldRespectBeanContract() {
        EqualsVerifier.forClass(DeletedMessageWithStorageInformation.class).verify();
    }

    @Test
    void constructorShouldThrowOnNullDeletedMessage() {
        assertThatThrownBy(() -> new DeletedMessageWithStorageInformation(null, STORAGE_INFORMATION))
            .isInstanceOf(NullPointerException.class);
    }

    @Test
    void constructorShouldThrowOnNullStorageInformation() {
        assertThatThrownBy(() -> new DeletedMessageWithStorageInformation(DeletedMessageFixture.DELETED_MESSAGE, null))
            .isInstanceOf(NullPointerException.class);
    }
}