/****************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/

package org.apache.james.blob.api;

import java.util.Objects;

public class BlobType {
    private final String name;
    private final BlobStore.StoragePolicy storagePolicy;

    public BlobType(String name, BlobStore.StoragePolicy storagePolicy) {
        this.name = name;
        this.storagePolicy = storagePolicy;
    }

    public String getName() {
        return name;
    }

    public BlobStore.StoragePolicy getStoragePolicy() {
        return storagePolicy;
    }

    @Override
    public final boolean equals(Object o) {
        if (o instanceof BlobType) {
            BlobType blobType = (BlobType) o;

            return Objects.equals(this.name, blobType.name)
                && Objects.equals(this.storagePolicy, blobType.storagePolicy);
        }
        return false;
    }

    @Override
    public final int hashCode() {
        return Objects.hash(name, storagePolicy);
    }
}
