/****************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/

package org.apache.james.lmtpserver.netty;

import org.apache.james.metrics.api.Metric;
import org.apache.james.metrics.api.MetricFactory;
import org.apache.james.smtpserver.netty.SmtpMetrics;

public class LMTPMetricsImpl implements SmtpMetrics {
    private static final String LMTP_CONNECTIONS = "lmtpConnections";
    private static final String LMTP_COMMANDS = "lmtpCommands";

    private final MetricFactory metricFactory;

    public LMTPMetricsImpl(MetricFactory metricFactory) {
        this.metricFactory = metricFactory;
    }

    @Override
    public Metric getConnectionMetric() {
        return metricFactory.generate(LMTP_CONNECTIONS);
    }

    @Override
    public Metric getCommandsMetric() {
        return metricFactory.generate(LMTP_COMMANDS);
    }
}
